@@ -107,6 +107,21 @@ sub module2path {
     return $path;
 }
 
+sub module2url {
+    my $module   = shift;
+    my $opt      = shift || {};
+    my $scheme   = $opt->{scheme} || 'http';
+    my $path     = exists $opt->{path} ? $opt->{path} : module2path($module);
+
+    module($module);
+
+    my $config   = Apache::Test::config();
+    my $hostport = hostport($config);
+
+    $path =~ s|^/||;
+    return "$scheme://$hostport/$path";
+}
+
 sub user_agent {
     my $args = {@_};
 
@@ -165,16 +180,19 @@ sub do_request {
 
 sub hostport {
     my $config = shift || Apache::Test::config();
-    local $config->{vars}->{scheme} =
-      $Apache::TestRequest::Scheme || $config->{vars}->{scheme};
+    my $vars = $config->{vars};
+    local $vars->{scheme} =
+        $Apache::TestRequest::Scheme || $vars->{scheme};
     my $hostport = $config->hostport;
 
+    my $default_hostport = join ':', $vars->{servername}, $vars->{port};
     if (my $module = $Apache::TestRequest::Module) {
-        $hostport = $config->{vhosts}->{$module}->{hostport}
-          unless $module eq 'default';
+        $hostport = $module eq 'default'
+            ? $default_hostport
+            : $config->{vhosts}->{$module}->{hostport};
     }
 
-    $hostport;
+    $hostport || $default_hostport;
 }
 
 sub resolve_url {
@@ -253,9 +271,7 @@ sub vhost_socket {
     local $Apache::TestRequest::Module = $module if $module;
 
     my $hostport = hostport(Apache::Test::config());
-    die "can't find hostport for '$module',\n",
-        "make sure that vhost_socket() was passed a valid module name"
-            unless defined $hostport;
+
     my($host, $port) = split ':', $hostport;
     my(%args) = (PeerAddr => $host, PeerPort => $port);
 
@@ -780,11 +796,29 @@ But note that redirection will B<not> work with C<POST> unless LWP is
 installed. It's best, therefore, to check C<have_lwp> before running
 tests that rely on a redirection from C<POST>.
 
+Sometimes it is desireable to have C<Apache::TestRequest> remember
+cookies sent by the pages you are testing and send them back to the
+server on subsequent requests. This is especially necessary when
+testing pages whose functionality relies on sessions or the presence
+of preferences stored in cookies.
+
+By default, C<LWP::UserAgent> does B<not> remember cookies between
+requests. You can tell it to remember cookies between request by
+adding:
+
+  Apache::TestRequest::user_agent(cookie_jar => {});
+
+before issuing the requests.
+
+
 =head1 FUNCTIONS
 
 C<Apache::TestRequest> exports a number of functions that will likely
 prove convenient for use in the majority of your request tests.
 
+
+
+
 =head2 Optional Parameters
 
 Each function also takes a number of optional arguments.
@@ -837,6 +871,11 @@ test server via C<UPLOAD()> and its friends.
 Sends a simple GET request to the Apache test server. Returns an
 C<HTTP::Response> object.
 
+You can also supply additional headers to be sent with the request by
+adding their name/value pairs after the C<url> parameter, for example:
+
+  my $res = GET $url, 'Accept-Language' => 'de,en-us,en;q=0.5';
+
 =head3 GET_STR
 
 A shortcut function for C<GET($uri)-E<gt>as_string>.
@@ -1053,6 +1092,70 @@ testing what options the Apache server supports. Consult the HTTPD 1.1
 specification, section 9.2, at
 I<http://www.faqs.org/rfcs/rfc2616.html> for more information.
 
+
+
+
+
+=head2 URL Manipulation Functions
+
+C<Apache::TestRequest> also includes a few helper functions to aid in
+the creation of urls used in the functions above.
+
+
+
+=head3 C<module2path>
+
+  $path = Apache::TestRequest::module2path($module_name);
+
+Convert a module name to a path, safe for use in the various request
+methods above. e.g. C<::> can't be used in URLs on win32. For example:
+
+  $path = Apache::TestRequest::module2path('Foo::Bar');
+
+returns:
+
+  /Foo__Bar
+
+
+
+
+=head3 C<module2url>
+
+  $url = Apache::TestRequest::module2url($module);
+  $url = Apache::TestRequest::module2url($module, \%options);
+
+Convert a module name to a full URL including the current
+configurations C<hostname:port> and sets C<module> accordingly.
+
+  $url = Apache::TestRequest::module2url('Foo::Bar');
+
+returns:
+
+  http://$hostname:$port/Foo__Bar
+
+The default scheme used is C<http>. You can override this by passing
+your preferred scheme into an optional second param. For example:
+
+  $module = 'MyTestModule::TestHandler';
+  $url = Apache::TestRequest::module2uri($module, {scheme => 'https'});
+
+returns:
+
+  https://$hostname:$port/MyTestModule__TestHandler
+
+You may also override the default path with a path of your own:
+
+  $module = 'MyTestModule::TestHandler';
+  $url = Apache::TestRequest::module2uri($module, {path => '/foo'});
+
+returns:
+
+  http://$hostname:$port/foo
+
+
+
+
+
 =head1 ENVIRONMENT VARIABLES
 
 The following environment variables can affect the behavior of
