@@ -1,21 +1,37 @@
+# Copyright 2001-2004 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 package Apache::TestTrace;
 
 use strict;
 use warnings FATAL => 'all';
 
 use Exporter ();
-use vars qw(@Levels @Utils @Subs @ISA @EXPORT $VERSION $Level $LogFH);
+use vars qw(@Levels @Utils @Level_subs @Util_subs
+            @ISA @EXPORT $VERSION $Level $LogFH);
 
 BEGIN {
     @Levels = qw(emerg alert crit error warning notice info debug);
     @Utils  = qw(todo);
-    @Subs   = map {($_, "${_}_mark", "${_}_sub")} (@Levels, @Utils);
+    @Level_subs = map {($_, "${_}_mark", "${_}_sub")} (@Levels);
+    @Util_subs  = map {($_, "${_}_mark", "${_}_sub")} (@Utils);
 }
 
 @ISA     = qw(Exporter);
-@EXPORT  = (@Subs);
+@EXPORT  = (@Level_subs);
 $VERSION = '0.01';
-use subs (@Subs);
+use subs (@Level_subs, @Util_subs);
 
 # default settings overrideable by users
 $Level = undef;
